function fem2rdme(fem,file,sdused)
%FEM2RDME Create input to RDME given FEM-structure.
%   FEM2RDME(FEM,FILE,SDUSED) writes to file FILE the arguments as requested 
%   by RDME.
%
%   Arguments related to the chemical network (reaction propensities, 
%   stoichiometric information and so on) are not treated 
%   here but must be constructed elsewhere.
%
%   Generalized subdomains (non-empty argument SDUSED). 
%   The generalized subdomain vector is given a default value of one. Other 
%   values can be defined by a special expression (rdme_sd) which should be 
%   defined on every subdomain, boundary and point. The global expression 
%   rdme_sdlevel should be defined as the lowest dimension that rdme_sd is 
%   defined for.
%
%   See also RDME2FEM, RDME.

% S. Engblom 2008-08-01

% extend mesh
if ~isfield(fem,'xmesh')
  fem.xmesh = meshextend(fem);
end

% fundamentals
[nodes,dofs] = xmeshinfo(fem.xmesh,'out',{'nodes' 'dofs'});
[Mspecies,Ncells] = size(nodes.dofs);
Ndofs = Ncells*Mspecies;
if Ndofs ~= flngdof(fem)
  warning(['Number of degrees of freedoms mismatch. ' ...
          'Check the boundary conditions.']);
end

% permutation of species
[foo,sp] = ismember(fem.equ.dim,nodes.names);

% permutation fem -> rdme
f2r = reshape(1:Ndofs,Mspecies,Ncells);
f2r = reshape(f2r(sp,:),[],1);

% create tspan (if any)
tspan = zeros(1,0);
if isfield(fem,'sol') && isfield(fem.sol,'tlist')
  tspan = fem.sol.tlist;
end

% u0
u0 = asseminit(fem);
u0 = reshape(u0.u(f2r),Mspecies,Ncells);

% assemble matrices
[D,M] = assemble(fem,'out',{'K','D'});

% the (lumped) mass matrix gives the element volume
vol = full(sum(M,2));

% explicitly invert the lumped mass matrix and filter the diffusion matrix
[i,j,s] = find(D);
diagsum1 = sum(diag(D)./vol);
s = -s./vol(i);
ixkeep = find(s > 0);
i = i(ixkeep); j = j(ixkeep); s = s(ixkeep);

% permute and rebuild
if2r(f2r) = 1:Ndofs;
[i,j,p] = find(sparse(if2r(i),if2r(j),1:numel(s)));
D = sparse(i,j,s(p),Ndofs,Ndofs);
d = full(sum(D,2));
D = D+sparse(1:Ndofs,1:Ndofs,-d);
diagsum2 = sum(d);

% check if the difference is too big
if abs(diagsum2-diagsum1) > abs(diagsum1)*0.10
  warning('Many off diagonal negative elements in D.');
end

% switch to absolute numbers
u0real = 6.022e23*vol'*u0(:);
u0 = round(6.022e23*reshape(vol,Mspecies,Ncells).*u0);

% check if the difference is too big
if abs(u0real-sum(u0(:))) > u0real*1e-2
  warning('The initial concentration is not accurately represented.');
end

% need only volume of unique cells
vol = vol(f2r);
vol = vol(1:Mspecies:end);

% data matrix -For instance this is the coordinate vector containing
% coordinates for each dof.
%data = nodes.coords(:,dofs.nodes(1:Mspecies:end));

% generalized subdomains
if nargin < 3
  sd = ones(1,Ncells);
else
  if ~isfield(fem,'globalexpr')
    error(['Variable rdme_sdlevel must be defined as a global' ...
      'expression to use predefined subdomains.']);
  end

  [i,j] = ismember(fem.globalexpr(1:2:end),'rdme_sdlevel');
  if ~i
    error(['Variable rdme_sdlevel must be defined as a ' ... 
	   'global expression to use predefined subdomains.']);
  end
  rdme_sdlevel = str2num(fem.globalexpr(2*j){1});

  sd = zeros(1,Ncells);
  els = xmeshinfo(fem,'out','elements');
  sdim = max(xmeshinfo(fem,'out','edim'));
  for edim = sdim:-1:rdme_sdlevel
    enodes = els{edim+1}.nodes;
    if edim == 0
      spoint = [0];
    else
      spoint = [zeros(edim,1) eye(edim)];
    end
    rdme_sd = posteval(fem,'rdme_sd','spoint',spoint,'edim',edim);
    sd(enodes(:,rdme_sd.elind(1:edim+1:end))) = rdme_sd.d;
  end
  sd = sd(dofs.nodes(1:Mspecies:end));
end

% write to file
save(file,'tspan','u0','D','vol','sd','Ncells','-mat');
