function U=rdme(tspan,u0,D,N,G,vol,sd,varargin)
%  MATLAB interface to rdme_solve.
%   U = RDME(TSPAN,U0,D,N,G,vol,sd) 
%   Returns the state matrix U at each output time given by TSPAN, with 
%   TSPAN = [T0 T1 ... TFINAL]. The Mex-file 'mexrdme' has to be    
%   compiled with the desired reaction specification before the call to 
%   RDME.
%
%   U = RDME(TSPAN,U0,D,N,G,vol,sd,OPTS,data,K,I,S)
%   Simulates as above but with the default options replaced by OPTS, 
%   passes the generalized data vector data and inline propensities in the 
%   matrices K, I, S. Using this syntax you may not need to write and compile 
%   any propensities provided that they are all defined as 'inline'.
%
%   Any optional input can be bypassed using an empty matrix.
%
%   Definitions:
%     Mspecies is the number of species in the system.
%     Ncells is the  number of cells in the system.
%     Mreactions is the total number of reactions.
%     M1 is the number of inline propensities.
%     Ndofs is defined as Mspecies*Ncells. 
%
%   The arguments are:
%
%   -TSPAN. A vector that lists all output times. The result is undefined if
%   tspan is descending.
%   
%   -U0. The initial state of the system. The order is important and should
%   be that dof #i is U0(i) and the dofs located in cell #j are U0(:,j). The
%   size of U0 must be (Mspecies,Ncells).
%   
%   -D. The macroscopic diffusion matrix. D(i,j) gives the diffusion rate
%   from dof #i to dof #j. The diagonal of D should be negative (or zero)
%   and the sum of any row must be zero. D must be a sparse matrix of size
%   (Ndofs,Ndofs).
%
%   -N. The stochiometric matrix. N(:,j) describes how reaction #j changes the
%   number of species. N must be a sparse matrix of size (Mspecies,Mreactions).
%
%   -G. The dependency graph. G(i,j) is nonzero if the executing reaction j
%   means that i needs to be re-evaluated. The first Mspecies columns of G
%   cover diffusion events. G must be a sparse matrix of size
%   (Mreactions,Mspecies+Mreactions).
%
%   -vol. The volume vector. vol(i) gives the volume of cell #i. The size of
%   vol must be Ncells.
%
%   -sd. The subdomain number vector. sd(i) is the subdomain of cell #i. The
%   vector can also be used to separate boundaries, line segments and
%   points.
%
%   -OPTS. Vector of predefined options. If OPTS(1) is nonzero RDME reports
%   the progress of the solver. OPTS(2) sets the random seed to the given
%   value and if NaN is given the seed is not altered. Default values for OPTS
%   are [0 NaN].
%
%   -data. The generalized data matrix, data(:,j) gives a data vector for cell
%   #j. The size of data must be (dsize,Ncells).
%
%   -K. The inline propensity coefficient matrix. K(:,i) gives the coefficients
%   to reaction #i. Where i is in [1,M1]. The size of K must be (3,M1). See the
%   inline propensity section for details.
%
%   -I. The inline propensity index matrix. I(:,i) gives the indices used by
%   reaction #i where i is in [1,M1]. The size of I must be (3,M1). See the
%   inline propensity section for details.
%
%   -S. The inline propensity subdomain matrix. S(:,i) lists all subdomains in
%   which the corresponding inline propensity is off. S must be a sparse matrix
%   of size (Ncells,M1). See the inline propensity section for details.
%
%   INLINE PROPENSITIES:
%
%   The inline propensities are defined by three vectors.
%   [k1,k2,k3], [i,j,k] and [s1,s2...sn] taken from the columns of K, I 
%   and S respectively.
%
%   If i=j the result is k1*x[i]*(x[i]-1)/2*vol + k2*x[k] + k3*vol.
%   else k1*x[i]*x[j]/vol + k2*x[k] + k3*vol.
%
%   Note that the inline propensity is automatically off (the result is 
%   zero) in subdomain zero. Additionally it is off in all subdomains 
%   listed in [s1,s2...sn].
%
%   TSPAN, U0, D, vol and sd can be genereted by FEM2RDME. (see the COMSOL 
%   interface)
%   
%   See also FEM2RDME, MAKE.

% J. Cullhed 2008-06-18

%% Mandatory arguments.

% Check the number of arguments
if nargin<7
   error('Too few arguments to rdme.')     
end

% 1) Check tspan.
tspan = reshape(tspan,1,[]);
if issparse(tspan)||~isa(tspan,'double') 
  error('Input time-span must be a double vector.');
elseif any(diff(tspan(:))<=0)||size(tspan,2)<2
  error('Input time-span must be an increasing vector.');
end

% 2) Check u0.
[Mspecies,Ncells]=size(u0);
Ndofs=Mspecies*Ncells;
if issparse(u0)||~isa(u0,'double')
  error('Initial state must be a double vector.');
elseif any(u0(:)<0)
  error('Initial state has negative elements.');
end

% 3) Check D.
D=D'; % need fast access to rows of D - see rdme.c.
Ddiag=diag(D);
if ~issparse(D)||~isa(D, 'double')
  error('Diffusion matrix must be sparse.');
elseif any(Ddiag>0)
  error('Diffusion matrix has positive diagonal elements.');
elseif any(size(D)~=Ndofs)
  error('Wrong size of diffusion matrix.');
elseif any(abs(sum(D,1))>1000*eps*abs(Ddiag)')
  error('Sum of rows criterion in diffusion matrix is violated.');
end

% 4) Check N.
[MM,Mreactions]=size(N);
if ~issparse(N)||~isa(N, 'double')
  error('Stochiometric matrix must be sparse.');
elseif MM~=Mspecies
  error('Wrong size of stochiometric matrix.');
end

% 5) Check G.
if ~issparse(G)||~isa(G,'double')
  error('Dependency graph must be sparse.');
elseif any(size(G)~=[Mreactions,Mreactions+Mspecies])
  error('Wrong size of dependency graph.');
end

% 8) Check vol.
vol=reshape(vol,1,[]);
if issparse(vol)||~isa(vol,'double')
  error('Volume must be a double vector.');
elseif any(vol<=0)
  error('Volume vector can not be negative.');
elseif size(vol,2)~=Ncells
  error('Wrong size of volume vector.');
end

% 10) Check sd.
sd=reshape(sd,1,[]);
if issparse(sd)||~isa(sd,'double')
  error('Subdomain must be a double vector.');
elseif size(sd,2)~=Ncells
  error('Wrong size of subdomain vector.');
elseif any(sd~=ceil(sd))
  error('Subdomain vector must be integer.');
end

%% Optional arguments.

% Default values
data=zeros(0,Ncells);
OPTS=[1,NaN];
K=zeros(3,0);
I=zeros(3,0);
S=sparse(zeros(3,0));

if nargin > 7
  if ~isempty(OPTS)
    OPTS=varargin{1};
  end
end

if nargin > 8
  if ~isempty(data)
    data=varargin{2};
  end
end

if nargin > 9
  if nargin < 12
    error('Must specify K,I and S');
  end
  K=varargin{3};
  I=varargin{4};
  S=varargin{5};
end

% Check data.
[dsize,NN]=size(data);
if issparse(data)||~isa(data,'double')
  error('Data matrix must be a double matrix.');
elseif NN~=Ncells
  error('Wrong size of data matrix.');
end

% Check OPTS.
if size(OPTS,2)~=2
  error('Option vector must have size 2.');
elseif issparse(OPTS)||~isa(OPTS,'double')
  error('Options must be a double vector.');
end

% Check K.
if issparse(K)||~isa(K,'double')
  error('Inline propensity coefficient must be a double matrix.');
elseif size(K,1)~=3
  error('Wrong size of inline propensity coefficient matrix.');
end

% Check I.
if issparse(I)||~isa(I,'double')
  error('Inline propensity index must be a double matrix.');
elseif any(size(I)~=size(K)) || size(I,2)>Mreactions
  error('Wrong size of inline propensity index matrix.');
elseif any(I(:)<1 | Mspecies<I(:))
  error('Species index out of bounds.');
end

% Check S.
if ~issparse(S)||~isa(S,'double')
  error('Inline subdomain matrix must be sparse.');
elseif size(S,2)~=size(I,2)
  error('Wrong size of inline subdomain matrix.');
end

%% Fire rocket.
U=mexrdme(tspan,u0,D,N,G,K,I,S,vol,data,sd,OPTS);
